import sys
import templ8.blessing
from templ8.divine import divine
from templ8.radio import radio
from templ8.genesis import genesis
from templ8.blessing import makedir

DEITY_PATH = "d8y"
DEF_BASEHTML_PATH = "basehtml"
DEF_REPLACE_PATH = "repl8ce"

DEF_INPUT = "input"
DEF_OUTPUT = "output"


def help():
    print(templ8.blessing.TEMPL8_ASCII + "\n\n")
    print("  help             Display this list.")
    print("  genesis [name]   Create a new templ8 project in a folder named [name].")
    print("  divine           Assemble a templ8 site.")
    print("  radio            Assemble a templ8 blog.")
    print("")


def main():
    if len(sys.argv) <= 1:
        help()
    elif sys.argv[1] == "help":
        help()
    elif sys.argv[1] == "divine":
        divine()
    elif sys.argv[1] == "radio":
        radio()
    elif sys.argv[1] == "genesis":
        if len(sys.argv) >= 3:
            genesis(sys.argv[2])
        else:
            raise Exception("Unexpected number of arguments")
    else:
        raise Exception("Unknown command")
    



import os
import templ8.blessing
from templ8.blessing import makedir

# Genesis of a website
def genesis(name):
    if not name.isidentifier():
        raise Exception("Name must only contain alphanumeric letters (a-z and 0-9) or underscores (_). Cannot start with a number or contain spaces.")
    makedir(name)
    makedir(os.path.join(name, templ8.blessing.DEF_INPUT))
    makedir(os.path.join(name, templ8.blessing.DEF_OUTPUT))
    open(os.path.join(name, templ8.blessing.DEITY_PATH), 'a').close()
    open(os.path.join(name, templ8.blessing.DEF_BASEHTML_PATH), 'a').close()
    open(os.path.join(name, templ8.blessing.DEF_REPLACE_PATH), 'a').close()
    
    print("Finished creating")


import os
from templ8.blessing import makedir

class ProgramState:
    def __init__(self):
        self.deity_path = r"d8y"
        self.basehtml_path = r"basehtml"
        self.replacements_path = r"repl8ce"
        self.txignore_path = r"txignore"
        self.replacements = {}

        self.input_folder = r"input"
        self.output_folder = r"output"
        
        
        # Load d8y
        if not os.path.exists(self.deity_path):
            raise Exception("No " + self.deity_path + " file found")
        
        # Core Renaming
        with open(self.deity_path, "r") as d8y_file:
            for line in d8y_file.readlines():
                keyval = line.split("=")
                if len(keyval) != 2:
                    raise Exception("Error in d8y file format:\n   " + line)
                if keyval[0] == "input" and keyval[1].isidentifier():
                    self.input_folder = keyval[1]
                elif keyval[0] == "output" and keyval[1].isidentifier():
                    self.output_folder = keyval[1]
                elif keyval[0] == "replace" and keyval[1].isidentifier():
                    self.replacements_path = keyval[1]
                elif keyval[0] == "basehtml" and keyval[1].isidentifier():
                    self.basehtml_path = keyval[1]
                elif keyval[0] == "txignore" and keyval[1].isidentifier():
                    self.txignore_path = keyval[1]
        
        
        self.input_folder = os.path.normpath(self.input_folder)
        self.output_folder = os.path.normpath(self.output_folder)
        self.replacements_path = os.path.normpath(self.replacements_path)
        self.basehtml_path = os.path.normpath(self.basehtml_path)
        self.txignore_path = os.path.normpath(self.txignore_path)

        if not os.path.exists(self.basehtml_path):
            raise Exception("No" + self.basehtml_path + "file found")

        self.basehtml_content = open(self.basehtml_path, "r").read()

        
        # Make input and output folder if they don't yet exist
        makedir(self.input_folder, "No input directory found, creating one")
        makedir(self.output_folder, "No output directory found, creating one")

        # Load the replacements from repl8ce
        if os.path.exists(self.replacements_path):
            replacement_text = open(self.replacements_path, "r").readlines()
            for i in replacement_text:
                if replacement_text != "":
                    rep_key = i.split("=")
                    if len(rep_key) == 2:
                        self.replacements[rep_key[0]] = rep_key[1]
                    elif len(rep_key) == 1:
                        self.replacements[rep_key[0]] = ""
                    else:
                        raise Exception("More than one value assignments on a replace key")
        else:
            print("WARNING: No " + self.replacements_path + " file found, continuing")

        
        # Load txignore
        self.txignore = []
        if os.path.exists(self.txignore_path):
            self.txignore = open(self.txignore_path, "r").readlines()


import os
import shutil
import textile
import templ8.programstate
import pypandoc
from templ8.blessing import makedir
from templ8.blessing import mod_replaces
from templ8.blessing import parse_content

# Divines a website
def divine():
    state = templ8.programstate.ProgramState()
    for subdir, dirs, files in os.walk(state.input_folder):
        for dir in dirs:
            path = os.path.join(subdir, dir).replace(state.input_folder, state.output_folder, 1)
            makedir(path)
            
        for file in files:
            path = os.path.join(subdir, file)
            file_extension = os.path.splitext(path)[1]
            outpath = path.replace(state.input_folder, state.output_folder, 1)
            outhtml = outpath.replace(file_extension, ".html", -1)
            if file_extension in [".textile", ".md"]:
                with open(path, "r") as f:
                    contents = ""
                    file_split = f.read().split("-BEGINFILE-")
                    file_headers = ""
                    file_content = ""
                    if len(file_split) == 2:
                        file_headers = file_split[0]
                        file_content = file_split[1]
                    elif len(file_split) == 1:
                        file_headers = ""
                        file_content = file_split[0]
                    else:
                        raise Exception("More than one -BEGINFILE- markers")
                    
                    # Check if the folder is in txignore
                    in_txignore = False
                    for i in state.txignore:
                        if os.path.join(state.input_folder, os.path.normpath(i)) == path or os.path.join(state.input_folder, os.path.normpath(i)) == subdir:
                            in_txignore = True
                            break
                          
                    
                    if not in_txignore:
                        filerepl = state.replacements.copy()
                        mod_replaces(filerepl, file_headers)
                        
                        
                        # Turn the content into HTML
                        contents = parse_content(file_content, file_extension)
                        
                        # Put the content in the base HTML
                        if not "CUSTOMBASE" in filerepl:
                            contents = state.basehtml_content.replace("##CONTENT##", contents)
                        else:
                            if os.path.exists(filerepl["CUSTOMBASE"]):
                                contents = open(filerepl["CUSTOMBASE"], "r").read().replace("##CONTENT##", contents)
                            else:
                                raise Exception(os.path.join(subdir, file) + " uses a CUSTOMBASE that doesn't exist")
                        
                        
                        for key in filerepl:
                            contents = contents.replace("##"+key+"##", filerepl[key])
                            
                        with open(outhtml, "w") as f:
                            f.write(contents)
                    else:
                        with open(outpath, "w") as f:
                            f.write(file_content)
                    
                    
                    
            elif not os.path.exists(outpath):
                shutil.copy(path, outpath)
    

    print("Finished assembling")


import os
import textile
import templ8.programstate
import templ8.blessing
from templ8.blessing import makedir
from templ8.blessing import mod_replaces
from templ8.blessing import parse_content

# Build the blog
def radio():
    state = templ8.programstate.ProgramState()
    blog_input = "blog"
    blog_output = os.path.join(state.output_folder, "blog")
    makedir(blog_input)
    makedir(blog_output)
    makedir(os.path.join(blog_output, "posts"))
    article_format_path = "blogbase"
    if not os.path.exists(article_format_path):
        with open(article_format_path, "w") as f:
            f.write(templ8.blessing.DEF_BASEHTML_CONTENT)
    
    # The article format is the file that defines how an article will look, both in its own page and in the index
    article_format_page = open(article_format_path, "r").read().split("-BEGININDEX-")[0]
    article_format_index = open(article_format_path, "r").read().split("-BEGININDEX-")[1]
    blog_replacekeys = open(article_format_path, "r").read().split("-BEGININDEX-")[2]
    
    blog_index = ""
    
    # Walk through the blog folder
    for subdir, dirs, files in os.walk(blog_input):
        sortedfiles = files
        sortedfiles.sort(reverse = True)
        for file in sortedfiles:
            path = os.path.join(subdir, file)
            file_extension = os.path.splitext(path)[1]
            file_replace = state.replacements
            # These refer to the individual articles
            file_headers = open(path, "r").read().split("-BEGINFILE-")[0]
            file_content = open(path, "r").read().split("-BEGINFILE-")[1]
            
            # Get the replace keys of the individual article
            mod_replaces(file_replace, file_headers)
            
            # This variable refers to the article page with the contents of this individual file
            article_page = article_format_page
            
            # Apply the file's keys to the article page and put in the content
            article_page = article_page.replace("##CONTENT##", file_content)
            
            for key in file_replace:
                article_page = article_page.replace("##"+key+"##", file_replace[key])
            
            
            # Get the keys and content of this article page
            article_page_headers = article_page.split("-BEGINFILE-")[0]
            article_page_content = article_page.split("-BEGINFILE-")[1]
            
            mod_replaces(file_replace, article_page_headers)
            
            # This variable is the final page
            final_page = state.basehtml_content
            
            # Apply the keys of the article page and put in the content
            final_page = final_page.replace("##CONTENT##", parse_content(article_page_content, file_extension))
            
            # (This var is for the index page, it takes the format and puts the info in)
            current_file_index = textile.textile(parse_content(article_format_index, file_extension))
            
            current_file_index = current_file_index.replace("##LINK##", 'posts/' + file.replace(".textile", "/index.html"))
            for key in file_replace:
                final_page = final_page.replace("##"+key+"##", file_replace[key])
                current_file_index = current_file_index.replace("##"+key+"##", file_replace[key])
            blog_index += current_file_index + "\n"
            
            # Save it
            blog_outpath = os.path.join(blog_output, "posts", file.replace(".textile", "/index.html"))
            makedir(os.path.join(blog_output, "posts", file.replace(".textile", "")))
            with open(blog_outpath, "w") as f:
                f.write(final_page)
    
    # Generate the index
    blog_replace = state.replacements.copy()
    mod_replaces(blog_replace, blog_replacekeys)
    
    index_html = state.basehtml_content
    if "CUSTOMBASE" in blog_replace:
        if os.path.exists(blog_replace["CUSTOMBASE"]):
            index_html = open(filerepl["CUSTOMBASE"], "r").read()
    
    index_html = index_html.replace("##CONTENT##", blog_index)
    for key in blog_replace:
        index_html = index_html.replace("##"+key+"##", blog_replace[key])
    
    with open(os.path.join(blog_output, "index.html"), "w") as f:
        f.write(index_html)
        
    print("Finished assembling")